@@ -24,7 +24,14 @@
 #include <Rinternals.h>
 #include <stdbool.h>
 
-SEXP read_nbt(SEXP r_rawval, SEXP r_max_elements);
-SEXP read_nbt_values(const unsigned char** ptr, const unsigned char* end, int max_elements, bool stop_on_end);
-SEXP write_nbt(SEXP r_value);
+SEXP read_nbt(SEXP r_rawval);
+SEXP read_nbt_values(const unsigned char** ptr, const unsigned char* end);
+SEXP read_nbt_value(const unsigned char** ptr, const unsigned char* end);
 
+SEXP write_nbt(SEXP r_value);
+R_xlen_t write_nbt_value(SEXP r_value, unsigned char** ptr, unsigned char* end);
+R_xlen_t write_nbt_values(SEXP r_value, unsigned char** ptr, unsigned char* end);
+
+#define return_nbt_error() { Rf_error("Malformed NBT data: at %s, line %d.",  __FILE__, __LINE__ ); return R_NilValue; }
+#define return_nbt_error0() { Rf_error("Malformed NBT data: at %s, line %d.",  __FILE__, __LINE__ ); return 0; }
+#define return_nbt_error_tag(x) { Rf_error("Malformed NBT data with tag `%d`: at %s, line %d.", (x), __FILE__, __LINE__ ); return R_NilValue; }
